/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#include <devices.h>

/*******************************************************************************
* File: main.c
*
* Version: 3.0
*
* Description:
*   This is a basic design source code.
*
********************************************************************************/

/* Variable declarations for DMA_1 */
uint8 DMA_1_Chan;
uint8 DMA_1_TD[ADC_SAR_Seq_1_NUMBER_OF_CHANNELS];

/* DMA Configuration for DMA_1 */
#define DMA_1_BYTES_PER_BURST 2	//Each TD handles a single channel, 1 16-bit value each
#define DMA_1_REQUEST_PER_BURST 1
#define DMA_1_SRC_BASE (CYDEV_SRAM_BASE)	//Regular ADCs result is DMA'd from the Peripheral Spoke. The sequencer ADC results are internally DMA'd to an array in SRAM. Results can be DMA'd from there.
#define DMA_1_DST_BASE (CYDEV_SRAM_BASE)

/* The number of samples of each channel to capture */
#define NumRepsToCapture 100 

/* Interrupt Flag */
volatile uint8 DMADone;

/*******************************************************************************
* Function Name: main
********************************************************************************
* Summary:
* Main function performs following functions:
* 1: Start the clock
* 2: Start the Timer
* 3: Start the interrupts
* 4: Start ADC DelSig and its interrupts
* 5: Testing for sample available from ADC
* 6: Storing the sample into the array
* 7: Comparing the samples
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/        

int main()
{
	/* Compare the values extracted from the ADC using DMA and using the API */
	static uint16 DMAADCResultArray[ADC_SAR_Seq_1_NUMBER_OF_CHANNELS][NumRepsToCapture] = {0};
	static uint16 manualADCResultArray[ADC_SAR_Seq_1_NUMBER_OF_CHANNELS] = {0};
	uint16 chan=0;
	
	DMADone = 0;	//When the TD completed the DMA transfer this flag is set by the ISR
	
    /* Enable global interrupts */
    CyGlobalIntEnable;
    
    /* Initializing DMA_1 */
    DMA_1_Chan = DMA_1_DmaInitialize(DMA_1_BYTES_PER_BURST, DMA_1_REQUEST_PER_BURST,
                                     HI16(DMA_1_SRC_BASE),  HI16(DMA_1_DST_BASE));

	//step through the TDs and allocate.
	//Do this in a seperate loop first so that all TDs are allocated before the TD setup loop. Otherwise pointing to the next TDs in the setup loop may cause a pointer to an uninitialised TD.
	for(chan = 0; chan < ADC_SAR_Seq_1_NUMBER_OF_CHANNELS; chan++)
	{
		DMA_1_TD[chan] = CyDmaTdAllocate();
		if(DMA_1_TD[chan] == DMA_INVALID_TD)
			return -1;
	}
	//step through and setup each TD
	for(chan = 0; chan < ADC_SAR_Seq_1_NUMBER_OF_CHANNELS; chan++)
	{	
		/* Configuring the TD for this DMA transfer. Transfer count should be 2 Bytes by number of samples */
		if(chan == (ADC_SAR_Seq_1_NUMBER_OF_CHANNELS-1))
    		// For the last TD/ADCChan, fire the output terminal to say we're finished
    		CyDmaTdSetConfiguration(DMA_1_TD[chan], 2*NumRepsToCapture, DMA_DISABLE_TD, DMA_1__TD_TERMOUT_EN | CY_DMA_TD_INC_DST_ADR );
		else
			CyDmaTdSetConfiguration(DMA_1_TD[chan], 2*NumRepsToCapture, DMA_1_TD[chan+1], CY_DMA_TD_INC_DST_ADR );
    
    	/* Configuring the TD address for the transfer. Sequencer ADC results are stored in ADC_SAR_Seq_1_finalArray. */
    	CyDmaTdSetAddress(DMA_1_TD[chan], LO16((uint32)&(ADC_SAR_Seq_1_finalArray[chan])), LO16((uint32)DMAADCResultArray[chan]));
    }
    /* Setting initial TD */
    CyDmaChSetInitialTd(DMA_1_Chan, DMA_1_TD[0]);

	/* ISR Start */
	isr_1_Start();
	
    /* Enable the DMA channel */
    CyDmaChEnable(DMA_1_Chan, 1);

	/* Start the ADC */
    ADC_SAR_Seq_1_Start();
	ADC_SAR_Seq_1_IRQ_Disable();	//we're not using the internal interrupt
	ADC_SAR_Seq_1_StartConvert();
	CyDelay(1000);	//give the ADC time to settle
	
    for(;;) {
		if(DMADone == 1)
		{
			//reset the DMA done flag
			DMADone = 0;
			// Grab the result values manually for comparison
			/* Channel results are saved in reverse order in the results array. Store them so they appear in the results array in the same order as the DMA data.*/
			for(chan = 0; chan < ADC_SAR_Seq_1_NUMBER_OF_CHANNELS; chan++)
			{
				manualADCResultArray[chan] = ADC_SAR_Seq_1_GetResult16(ADC_SAR_Seq_1_NUMBER_OF_CHANNELS-chan-1);
			}

			/* Enable the DMA channel again */
    		CyDmaChEnable(DMA_1_Chan, 1);
		}
	}

	return 0;
}

/* [] END OF FILE */
